/*
 * StubTask.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file StubTask.hh
 *
 * Header file and implementation for the StubTask class.
 */

#ifndef _stub_task_hh
#define _stub_task_hh

#include "BrokerS.h"

/**
 * A Broker::Task class that can be used for testing.
 */
class StubTask : public POA_Broker::Task
{

public:

    /**
     * Construct an empty StubTask object.
     */
    StubTask()
    {
    };

    /**
     * Deconstruct a StubTask object.
     */
    virtual ~StubTask()
    {
    };

    /**
     * @return The string "<stub>".
     */
    virtual char *Name()
	throw (CORBA::SystemException)
    {
	return CORBA::string_dup("<stub>");
    };

    /**
     * Empty implementation of Broker::Task::BeginCPUScheduling.
     *
     * @param manager Unused.
     * @param sp Unused.
     */
    virtual void BeginCPUScheduling(Broker::Manager_ptr manager,
				    const Broker::ScheduleParameters &sp)
	throw (CORBA::SystemException,
	       Broker::DuplicateScheduleParameter,
	       Broker::InvalidScheduleParameter,
	       Broker::MissingScheduleParameter)
    {
	this->st_Manager = Broker::Manager::_duplicate(manager);
    };

    /**
     * Empty implementation of Broker::Task::EndCPUScheduling.
     */
    virtual void EndCPUScheduling(void)
	throw (CORBA::SystemException)
    {
	this->st_Manager = Broker::Manager::_nil();
    };

protected:

    /**
     * The Broker::Manager used to handle scheduling during contention.  Only
     * valid between #BeginCPUScheduling and #EndCPUScheduling calls.
     */
    Broker::Manager_var st_Manager;
    
};

#endif
