/*
 * manager_good.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file manager_good.cc
 *
 * Test the ManagerImpl class.
 */

#include "config.h"

#if !defined(DEBUG)
#define DEBUG 1
#endif

#include <assert.h>
#include <assert_pp.h>

#include <BrokerC.h>
#include <ManagerImpl.hh>

#include <StubTask.hh>
#include <StubPolicy.hh>

int main(int argc, char *argv[])
{
    int retval = EXIT_SUCCESS;
    
    /* ORB setup */
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
    
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    
    PortableServer::POA_var root_poa =
	PortableServer::POA::_narrow(obj.in());
    PortableServer::POAManager_var mgr = root_poa->the_POAManager();
    mgr->activate();

    /* Broker setup */
    ManagerImpl *mi = new ManagerImpl("dummy");
    Broker::Manager_var manager;

    manager = mi->_this();

    StubPolicy *sp = new StubPolicy("dummy");
    Broker::Policy_var policy;

    policy = sp->_this();

    StubTask *st = new StubTask();
    Broker::Task_var task;

    task = st->_this();

    /* Check initial values */
    assert(CORBA::is_nil(manager->CurrentPolicy()));

    Broker::ScheduleParameters sparam;
    sparam.length(0);
    
    /* Add a task and */
    manager->AddTask(task.in(), sparam);
    /* ... make sure its in there (i.e. no exception). */
    manager->RemoveTask(task.in());

    /* Add policy and */
    manager->CurrentPolicy(policy.in());
    /* ... check activated state */
    assert(!CORBA::is_nil(manager->CurrentPolicy()));
    assert(sp->sp_Tasks.length() == 0);

    /* Make sure we can set a nil policy */
    manager->CurrentPolicy(Broker::Policy::_nil());
    assert(CORBA::is_nil(manager->CurrentPolicy()));

    /* Make sure existing tasks are passed to Broker::Policy::Activate */
    manager->AddTask(task.in(), sparam);
    manager->CurrentPolicy(policy.in());
    assert(sp->sp_Tasks.length() == 1);

    /* Make sure Broker::Policy::Deactivate is called and */
    manager->CurrentPolicy(Broker::Policy::_nil());
    /* ... the policy drops the tasks. */
    assert(sp->sp_Tasks.length() == 0);

    /* Recheck activation. */
    manager->CurrentPolicy(policy.in());
    assert(sp->sp_Tasks.length() == 1);

    /* Check dynamic removal and addition of task. */
    manager->RemoveTask(task.in());
    assert(sp->sp_Tasks.length() == 0);
    manager->AddTask(task.in(), sparam);
    assert(sp->sp_Tasks.length() == 1);

    task = Broker::Task::_nil();
    delete st;
    
    //policy = Broker::Policy::_nil();
    //delete sp;

    manager = Broker::Manager::_nil();
    delete mi;
    
    return( retval );
}
