/*
 * RealTimeSchedule.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file RealTimeSchedule.hh
 *
 * Header file for the RealTimeSchedule helper class.
 */

#ifndef _cpu_schedule_hh
#define _cpu_schedule_hh

#include "BrokerC.h"

/**
 * The RealTimeSchedule class is used to help policies that manage
 * Broker::RealTimeTask's.
 */
class RealTimeSchedule
{

public:

    /**
     * Construct a RealTimeSchedule object with the given values.
     *
     * @param start The start time for the task.
     * @param period The task period in microseconds.
     * @param deadline The task deadline in microseconds, relative to the start
     *                 of the period.
     */
    RealTimeSchedule(CORBA::ULong start = 0,
		     CORBA::ULong period = 0,
		     CORBA::ULong deadline = 0);

    /**
     * Deconstruct a RealTimeSchedule object.
     */
    virtual ~RealTimeSchedule();

    /**
     * Compute the number of microseconds that this schedule will have to
     * execute during the given time frame.
     *
     * @param start The start of the time frame, in microseconds.
     * @param stop The end of the time frame, in microseconds.
     * @return The number of microseconds during the time frame in which this
     *         task will be schedulable.
     */
    CORBA::ULong intersect(CORBA::ULong start, CORBA::ULong stop);

    /** The start time for the task.  XXX somewhat bogus at the moment */
    CORBA::ULong rts_Start;
    
    /** The task period in microseconds. */
    CORBA::ULong rts_Period;
    
    /**
     * The task deadline in microseconds, relative to the start of the
     * period.
     */
    CORBA::ULong rts_Deadline;

};

#endif
