/*
 * rk_util.h
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file rk_util.h
 *
 * Header file for utility functions related to the TimeSys resource kernel.
 */

#ifndef _rk_util_h
#define _rk_util_h

#ifdef __cplusplus
extern "C" {
#endif

/*
 * A structure used to hold the results of a rk_resource_set_get_usage call.
 *
 * proc_count - The number of processes in the resource set at the time of
 * sampling.
 * cpu_usage - The total CPU usage, in microseconds, for all of the processes
 * in the set.
 */
typedef struct rk_resource_set_usage {
    unsigned int proc_count;
    unsigned long long cpu_usage;
} *rk_resource_set_usage_t;

/**
 * Search for a resource set by name and return the first one encountered.
 *
 * @param name The resource set name to search for.
 * @return A valid resource set pointer if the name was found, otherwise,
 * NULL_RESOURCE_SET.
 */
rk_resource_set_t rk_resource_set_get_by_name(const char *name);

/**
 * Compute the total resource usage for all the processes in a resource set.
 * Currently, this is implemented by stepping through all of the processes
 * attached to the resource set and pulling the usage information from
 * '/proc/@<pid@>/stat'.  Unfortunately, the standard kernel does not provide
 * high-resolution data in the 'stat' file, so the file in the 'patch'
 * directory before this function can be used.
 *
 * @sa rk_util/patch/tslinux-3.1-procstat.patch
 *
 * @param rs The resource set to compute the resource usage for.
 * @param rsu An allocated rk_resource_set_stat_t object where the results
 * should be placed.
 * @return Zero on success, otherwise, an errno value.
 */
int rk_resource_set_get_usage(rk_resource_set_t rs,
			      rk_resource_set_usage_t rsu);

#ifdef __cplusplus
}
#endif
    
#endif
