/*
 * rk_util.c
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file rk_util.c
 *
 * Implementation of the functions in rk_util.h.
 */

#include "config.h"

#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>

#include "assert_pp.h"
#include "instrumentation.h"

#include <rk.h>
#include "rk_util.h"

/* BEGIN Instrumentation data */

#if defined(INSTR_rk_resource_set_get_usage_data)
static struct iPoint INSTR_get_point = {
    INSTR_rk_resource_set_get_usage_data,
};
#endif

#if defined(INSTR_rk_proc_usage_data)
static struct iPoint INSTR_rk_proc_usage_point = {
    INSTR_rk_proc_usage_data,
};
#endif

/* END Instrumentation data */

rk_resource_set_t rk_resource_set_get_by_name(const char *name)
{
    rk_resource_set_t retval = NULL_RESOURCE_SET;
#define MAX_RK_SETS 128
    rk_resource_set_t rk_sets[MAX_RK_SETS];
    char rs_name[RK_NAME_LEN + 1];
    int lpc, actual;

    require(name != NULL);
    
    actual = rk_resource_sets_get_list(rk_sets, MAX_RK_SETS);
    for( lpc = 0; (lpc < actual) && (retval == NULL_RESOURCE_SET); lpc++ )
    {
	rs_name[0] = '\0';
	if( rk_resource_set_get_name(rk_sets[lpc], rs_name) == 0 )
	{
	    if( strcmp(rs_name, name) == 0 )
	    {
		retval = rk_sets[lpc];
	    }
	}
    }
#undef MAX_RK_SETS
    return( retval );
}

int rk_resource_set_get_usage(rk_resource_set_t rs,
			      rk_resource_set_usage_t rsu)
{
    int retval = 0;

    if( (rs == NULL) || (rsu == NULL) )
    {
	retval = EINVAL;
    }
    else
    {
#define MAX_PID_COUNT 128
	INSTR_rk_resource_set_get_usage(&INSTR_get_point, ({
	    static pid_t procs[MAX_PID_COUNT];
    
	    int lpc;
	    
	    rsu->proc_count = rk_resource_set_get_proclist(rs,
							   procs,
							   MAX_PID_COUNT);
	    rsu->cpu_usage = 0;
	    for( lpc = 0; (lpc < rsu->proc_count) && (retval == 0); lpc++ )
	    {
		static char filename[1024]; // XXX
		
		char *unused_string = filename;
		unsigned long unused_ulong;
		long unused_long;
		char unused_char;
		int unused_int;
		FILE *file;
		
		snprintf(filename,
			 sizeof(filename),
			 "/proc/%d/stat",
			 procs[lpc]);
		if( (file = fopen(filename, "r")) != NULL )
		{
		    unsigned long long usage = 0;
		    struct timeval utime, stime;
		    int scan_count;
		    
		    memset(&utime, 0, sizeof(utime));
		    memset(&stime, 0, sizeof(stime));
		    {
			scan_count = fscanf(file,
					    "%d %s %c %d %d %d %d %d %lu %lu %lu %lu %lu %lu %lu %ld %ld %ld %ld %ld %ld %lu %lu %ld %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %d %d %ld %ld %ld %ld",
					    &unused_int,
					    unused_string,
					    &unused_char,
					    &unused_int,
					    &unused_int,
					    &unused_int,
					    &unused_int,
					    &unused_int,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_long,
					    &unused_long,
					    &unused_long,
					    &unused_long,
					    &unused_long,
					    &unused_long,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_long,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_ulong,
					    &unused_int,
					    &unused_int,
					    &utime.tv_sec,
					    &utime.tv_usec,
					    &stime.tv_sec,
					    &stime.tv_usec);

			if( scan_count != 43 )
			{
			    retval = ENOSYS;
			}
		    }
		    usage += utime.tv_sec * 1000000;
		    usage += utime.tv_usec;
		    usage += stime.tv_sec * 1000000;
		    usage += stime.tv_usec;
		    INSTR_rk_proc_usage(&INSTR_rk_proc_usage_point, usage);
		    rsu->cpu_usage += usage;
		    fclose(file);
		}
	    }
	}));
    }
    return( retval );
}
