/*
 * rk_util_test.h
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file rk_util_test.c
 *
 * Test cases for the functions in rk_util.h.
 */

#include "config.h"

#ifndef DEBUG
#define DEBUG
#endif

#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

#include <assert_pp.h>
#include <instrumentation.h>

#include <rk/rk.h>
#include "rk_util.h"

int main(int argc, char *argv[])
{
    int retval = EXIT_FAILURE;
    rk_resource_set_t rs;

    instrumentation_data.iid_OutputFileName = "-";
    atexit(iPrintPointsAtExit);

    if( (rs = rk_resource_set_create("rk_util_test")) != NULL_RESOURCE_SET )
    {
	struct rk_resource_set_usage rsu;
	
	require(rk_resource_set_get_by_name("rk_util_test") !=
		NULL_RESOURCE_SET);
	require(rk_resource_set_get_by_name("non-existent") ==
		NULL_RESOURCE_SET);

	rsu.proc_count = 1;
	rsu.cpu_usage = 1;
	if( rk_resource_set_get_usage(rs, &rsu) == 0 )
	{
	    ensure(rsu.proc_count == 0);
	    ensure(rsu.cpu_usage == 0);
	}
	if( rk_resource_set_attach_process(rs, getpid()) == 0 )
	{
	    unsigned long long last_usage = 0;
	    int lpc;

	    retval = EXIT_SUCCESS;
	    for( lpc = 0; lpc < 1000; lpc++ )
	    {
		require(rk_resource_set_get_usage(rs, &rsu) == 0);
		require(rsu.proc_count == 1);
		
		if( rsu.cpu_usage < last_usage )
		{
		    retval = EXIT_FAILURE;
		}
		last_usage = rsu.cpu_usage;
	    }
	}
	else
	{
	    perror("rk_resource_set_attach_process");
	}
	rk_resource_set_destroy(rs);
    }
    else
    {
	perror("rk_resource_set_create");
    }
    return( retval );
}
