#! /bin/sh

RKTIMES=../src/rktimes

$RKTIMES -h > usage.out 2>&1

# Check for no arguments.
run_test $RKTIMES

check_output "rktimes works with no arguments?" <<EOF
$RKTIMES: Missing utility to monitor
`cat usage.out`
EOF

# Check for -n with a nonexistent set.
run_test $RKTIMES -n nonexistent

check_output "rktimes works with a bad resource set name?" <<EOF
$RKTIMES: No such resource set: nonexistent
`cat usage.out`
EOF

# Check for -o without an argument.
run_test $RKTIMES -o

check_output "-o with no argument works?" <<EOF
$RKTIMES: option requires an argument -- o
`cat usage.out`
EOF

# Check for -n without an argument.
run_test $RKTIMES -n

check_output "-n with no argument works?" <<EOF
$RKTIMES: option requires an argument -- n
`cat usage.out`
EOF

# Check for -n with a long argument.
run_test $RKTIMES -n 12345678901234567

check_output "-n with long argument works?" <<EOF
$RKTIMES: -n value too long (only 16 characters allowed)
`cat usage.out`
EOF

# Check for -p without an argument.
run_test $RKTIMES -p

check_output "-p with no argument works?" <<EOF
$RKTIMES: option requires an argument -- p
`cat usage.out`
EOF

# Check for -p with a non numeral argument.
run_test $RKTIMES -p foo

check_output "-p works with string?" <<EOF
$RKTIMES: Invalid -p value: foo
`cat usage.out`
EOF

# Check for -p 0.
run_test $RKTIMES -p 0

check_output "-p works with 0?" <<EOF
$RKTIMES: Invalid -p value: 0
`cat usage.out`
EOF

# Check for -p 65536.
run_test $RKTIMES -p 65536

check_output "-p works with 65536?" <<EOF
$RKTIMES: Invalid -p value: 65536
`cat usage.out`
EOF

# Check -V
run_test $RKTIMES -V

check_output "-V does not work?" <<EOF
$PACKAGE_VERSION
EOF

run_test $RKTIMES -C 50000

check_output "Compute time can be given without period?" <<EOF
$RKTIMES: Period time _must_ be specified with compute time
`cat usage.out`
EOF

run_test $RKTIMES -P 10000

check_output "Period can be given without compute time?" <<EOF
$RKTIMES: Compute time _must_ be specified with period
`cat usage.out`
EOF

run_test $RKTIMES -C 50000 -P 10000

check_output "Compute time can be bigger than period?" <<EOF
$RKTIMES: Compute time _must_ be less than period
`cat usage.out`
EOF
