/*
 * LoggingAdvocate.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file LoggingAdvocate.hh
 *
 * Header file for the LoggingAdvocate class.
 */

#ifndef _logging_advocate_hh
#define _logging_advocate_hh

#include <iostream>

#include <RealTimeTaskDelegateImpl.hh>

/**
 * A Broker::RealTimeAdvocate that suggests the task gets the exact amount of
 * compute time as needed by the last period.
 */
class LoggingAdvocate : public virtual RealTimeTaskDelegateImpl
{

public:

    LoggingAdvocate(void);

    virtual ~LoggingAdvocate(void);

    virtual void SetDelegateAttribute(const char *id,
				      const CORBA::Any &value)
	throw (CORBA::SystemException);

    virtual void ReportCPU(Broker::RealTimeTask_ptr rtt,
			   CORBA::ULong status,
			   CORBA::ULong advice)
	throw (CORBA::SystemException);

protected:

    /**
     * The time that this object was constructed.
     */
    struct timeval eta_StartTime;

    /**
     * The last time that the CPU usage was logged.
     */
    struct timeval eta_LastCPULog;

    /**
     * The output stream for the CPU usage log.
     */
    ofstream *eta_CPULog;

    /**
     * The last time the advice was logged.
     */
    struct timeval eta_LastAdviceLog;

    /**
     * The output stream for the advice log.
     */
    ofstream *eta_AdviceLog;

    /**
     * The output stream for the completion log.
     */
    ofstream *eta_CompleteLog;

};

#endif
