/*
 * MaxDecayTaskAdvocate.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file MaxDecayTaskAdvocate.hh
 *
 * Header file for the MaxDecayTaskAdvocate class.
 */

#ifndef _max_decay_task_advocate_hh
#define _max_decay_task_advocate_hh

#include <RealTimeTaskDelegateImpl.hh>

/**
 * An ExactTaskAdvocate that will always advise the maximum required processing
 * time it has ever seen.
 */
class MaxDecayTaskAdvocate : public virtual RealTimeTaskDelegateImpl
{

public:
    
    MaxDecayTaskAdvocate(void);

    virtual ~MaxDecayTaskAdvocate(void);
    
    /**
     * Compute a new processing time advice from the given status value.  The
     * max implementation will compare the advice reported by the parent class
     * against the current processing time and return the maximum of the two.
     *
     * @param rtt The task object that was actually added to the manager.
     * @param status The CPU usage of the task in microseconds.
     * @param advice The amount of CPU time, in microseconds, that the
     * application would like for the next period.
     * @return A processing time that will satisfy all requirements that have
     *         been encountered so far.
     * @exception CORBA::BAD_INV_ORDER if the method is called without
     *            BeginCPUScheduling() being called first.
     */
    virtual void ReportCPU(Broker::RealTimeTask_ptr rtt,
			   CORBA::ULong status,
			   CORBA::ULong advice)
	throw (CORBA::SystemException);

private:

    /**
     * The last advice reported by this advocate.
     */
    CORBA::ULong md_LastAdvice;

    /**
     * The maximum observed usage that is still below the last advice given.
     */
    CORBA::ULong md_SubMax;

    /**
     * The number of periods left before the reservation will be dropped to the
     * value of md_SubMax.
     */
    unsigned int md_Decay;
    
};

#endif
